//****************************************************************************
// Copyright (c) 2001 Microsoft Corporation.
//
// @File: ExpressionEval.cs
//
// Purpose: 
//  
// Notes:
//
// History:
//
//     @Version:
//     140123 KXH  04/25/03 Implement validation in Expression task.
//     139040 KXH  04/23/03 A task for evaluating expressions.  This is a 
//                          utility task and is
//
// @EndHeader@
//****************************************************************************

using System;
using Microsoft.SqlServer.Dts.Runtime.Wrapper;

// This class is here specifically to isolate and simplify the portion of the task
// that is dependent on the wrapper assembly for the runtime and to 
// eliminate type clashes between the wrapper and the managed assemblies.
namespace SSIS2K5.Samples.Tasks.ExpressionTask
{
	/// <summary>
	/// Summary description for ExpressionEval.
	/// </summary>
	public class ExpressionEval
	{
		private Microsoft.SqlServer.Dts.Runtime.Wrapper.ExpressionEvaluatorClass m_Expression;
		private string m_ExpressionString;

		public ExpressionEval()
		{
			try
			{
				m_Expression = new ExpressionEvaluatorClass();
			}
			catch (Exception e)
			{	
				System.Diagnostics.EventLog.WriteEntry( this.ToString(), e.Message );
				//ST_ASSERT(e.Message)
			}
		}

		public string Expression
		{
			get {return m_ExpressionString;}
			set {m_ExpressionString = value;}			
		}

		// Throws to caller if there is an error
		public object Evaluate(IDTSVariableDispenser90 vars)
		{
			object result = null;
			try
			{
				m_Expression.Expression = m_ExpressionString;
				m_Expression.Evaluate(vars, out result, true);
			}
			catch (Exception e)
			{
				System.Diagnostics.EventLog.WriteEntry(this.ToString(), e.Message);				
				throw e;
				//ST_ASSERT(e.Message)
			}
			return result;			
		}

		// Throws to caller if there was an error
		public object Validate(IDTSVariableDispenser90 vars)
		{
			object result = null;
			try
			{
				m_Expression.Expression = m_ExpressionString;
				m_Expression.Evaluate(vars, out result, false);
			}
			catch (Exception e)
			{
				System.Diagnostics.EventLog.WriteEntry(this.ToString(), e.Message);				
				throw e;
				//ST_ASSERT(e.Message)
			}
			return result;
		}

		public bool IsBoolean(IDTSVariableDispenser90 vars)
		{
			bool isBoolean = false;
			try
			{
				m_Expression.Expression = m_ExpressionString;
				m_Expression.IsBooleanExpression(vars, out isBoolean);
			}
			catch (Exception e)
			{
				System.Diagnostics.EventLog.WriteEntry(this.ToString(), e.Message);
				//ST_ASSERT(e.Message)
			}
			return isBoolean;			
		}
	}
}
