using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SqlServer.Dts.Runtime;


namespace SSIS2K5.Samples.Tasks.HTTPTask
{
   [
   DtsTask(
      DisplayName = "HTTPTask",
      Description = "A task for downloading files via HTTP",
      TaskContact = "HTTPTask, Copyright  Kirk Haselden",
      IconResource="SSIS2K5.Samples.Tasks.HTTPTask.TaskIcon.ico"
      )
   ]
   public class HTTPTask : Task
   {
      #region PRIVATES
      // Store a reference to connections
      private Connections m_connections;

      // Source and Dest Connection Manager IDs
      private string m_httpCMID = "";
      private string m_fileCMID = "";
      #endregion

      #region PROPERTIES
      // DestinationConnection manager
      public string FileConnectionManager
      {
         get
         {
            return GetConnectionName(m_connections, m_fileCMID);
         }
         set
         {
            m_fileCMID = GetConnectionID(m_connections, value);
         }
      }

      // SourceConnection manager
      public string HTTPConnectionManager
      {
         get
         {
            return GetConnectionName(m_connections, m_httpCMID);
         }
         set
         {
            m_httpCMID = GetConnectionID(m_connections, value);
         }
      }
      #endregion

      #region HELPERMETHODS
      // A simplified method for logging.
      // Does not require the additional overwritten parameters.
      private void WriteLog(IDTSLogging log, string logEventName, string message)
      {
         if (log != null && log.Enabled)
         {
            byte[] nothing = null;
            log.Log(logEventName, null, null, null, null, null, message, DateTime.Now, DateTime.Now, 0, ref nothing);
         }
      }
      #endregion

      #region TASKMETHODS
      // Get the connection reference for the connection manager properties
      public override void InitializeTask(Connections connections, VariableDispenser variableDispenser, IDTSInfoEvents events, IDTSLogging log, EventInfos eventInfos, LogEntryInfos logEntryInfos, ObjectReferenceTracker refTracker)
      {
         m_connections = connections;
      }

      // Validate the task settings. Simple validation done here. For production task, should validate connection manager type etc.
      public override DTSExecResult Validate(Connections connections, VariableDispenser variableDispenser, IDTSComponentEvents componentEvents, IDTSLogging log)
      {
         DTSExecResult execResult = DTSExecResult.Success;

         // Validate that the source connection exists and is valid.
         if (m_httpCMID.Length == 0)
         {
            componentEvents.FireError(100, "", "There is no source HTTP connection manager", "", 0);
         }
         
         // Validate that the destination connection exists and is valid.
         if (m_fileCMID.Length == 0)
         {
            componentEvents.FireError(100, "", "There is no destination File connection manager", "", 0);
         }

         return execResult;         
      }

      // Download the file.
      public override DTSExecResult Execute(Connections connections, VariableDispenser variableDispenser, IDTSComponentEvents componentEvents, IDTSLogging log, object transaction)
      {
         // Create an HttpClientConnection and use it to download 
         // a file from the location the connection manager specifies

         // Get the connection object from the HTTP Connection Manager
         Microsoft.SqlServer.Dts.Runtime.HttpClientConnection httpConnection;
         Object temp;

         // Setup for success
         DTSExecResult execResult = DTSExecResult.Success;

         // Try to get the connection
         try
         {
            // Get the http connection object from the HTTP Connection Manager.
            temp = connections[m_httpCMID].AcquireConnection(null);
            httpConnection = new HttpClientConnection(temp);

            // Get the server URL
            string url = httpConnection.ServerURL;
            string file = connections[m_fileCMID].AcquireConnection(null).ToString();

            // Ignore fire again. We only fire this once per execution.
            Boolean fireAgain = true;

            // Log information
            componentEvents.FireInformation( 1, "", "Downloading : " + url + " to the file : " + file, "", 0, ref fireAgain );

            // Do the download
            httpConnection.DownloadFile(file, true);
         }
         catch(Exception ex)
         {
            // Catch the exception and change the execution result
            componentEvents.FireError( 1, "", ex.Message, "", 0);
            execResult = DTSExecResult.Failure;
         }
         return execResult;
      }
      #endregion
   }
}

