//**********************************************************************************************
// Copyright (c) 2005, 2006 Kirk Haselden.
//
// File:                      StarterTask.cs
// Purpose:               Provide a way to set the value of parent variables
//                             in the package at any point in workflow.
// DATE:                  1/18/2006 
// PROJECT:		       SSIS2K5 Samples
// PROGRAMMER:   Kirk Haselden
// COMMENTS:        
// TABS:		             3/3 retain tabs
//
//**********************************************************************************************

#region using directives
using System;
using System.Text;
using System.Xml;
using Microsoft.SqlServer.Dts.Runtime;

#endregion

namespace SSIS2K5.Samples.Tasks.StarterTask
{
   [
   DtsTask(
		TaskType = "DTS90",
      DisplayName="Starter Task",
      IconResource="SSIS2K5.Samples.Tasks.StarterTask.TaskIcon.ico",
      Description="A sample task for starting custom tasks",
      UITypeName = "SSIS2K5.Samples.Tasks.StarterTask.StarterTaskUI, SSIS2K5.Samples.StarterTaskUI, Version=1.0.0.1, Culture=Neutral, PublicKeyToken=844aba2619516093",
		TaskContact = "Starter Task; Microsoft SQL Server Integration Services 2005, By Kirk Haselden; For Version 9.0 of Integration Services; (c) 2006 Kirk Haselden;http://www.CustomTask.com/StarterTask")
   ]
   public sealed class StarterTask : Task, IDTSComponentPersist
   {
      #region Task Specifics
      
      #region Private Members
         private string m_privateString;
         private int m_privateInt;
      #endregion

      #region Public Properties

      // A string property
      public string StringValue
      {
         get
         {
            return m_privateString;
         }
         set
         {
            m_privateString = value;
         }
      }

      // An int property
      public int IntValue
      {
         get
         {
            return m_privateInt;
         }
         set
         {
            m_privateInt = value;
         }
      }
      #endregion

      #endregion

      #region Task Section
      #region InitializeTask
      public override void InitializeTask(Connections connections, VariableDispenser variableDispenser, IDTSInfoEvents events, IDTSLogging log, EventInfos eventInfos, LogEntryInfos logEntryInfos, ObjectReferenceTracker refTracker)
		{         
         if (null != logEntryInfos)
			{				
            // Add a log entry info so users can filter the log entries of this type in the designer.
				//logEntryInfos.Add(LOGEVENTNAMES[(int)ENUMLOG.TESTLOGENTRY], StarterTask.StarterTaskLogEntryDescription, DTSLogEntryFrequency.Occasional);
			}
		}
		#endregion

      // Task Validate method for checking if needed resources are available.
      // Ideally, if the task passes validation, it will always execute successfully, 
      // or at least with succeed with regard to finding needed resources.
		#region Validate
		public override DTSExecResult Validate(Connections connections, VariableDispenser variableDispenser, IDTSComponentEvents events, IDTSLogging log)
		{
         DTSExecResult result = DTSExecResult.Success;

         // Check if required resources like variables and connections are available.
         // Do stuff here...

         return result;
		}
		#endregion

		#region Execute
		public override DTSExecResult Execute(Connections connections, VariableDispenser variableDispenser, IDTSComponentEvents events, IDTSLogging log, object transaction)
		{
         DTSExecResult result = DTSExecResult.Success;
         Boolean fireAgain = true;

         try
         {
            // Fire starting progress event.
            events.FireProgress("Starting to execute the StarterTask", 0, 0, 0, "", ref fireAgain);

            // Write some information about the execution.
            WriteLog(log, LOGEVENTNAMES[(int)ENUMLOG.TESTLOGENTRY], "This is a StartTask test log entry");

            // Fire completion progress event.
            // This is the minimum progress event logging that should always be done.
            events.FireProgress("Completed execution of the StarterTask", 100, 0, 0, "", ref fireAgain);
         }
         catch( Exception e)
         {
            if (null != events)
            {
               events.FireError(0, "", e.Message, "", 0);
            }
            result = DTSExecResult.Failure;
         }
			return result;
		}
		#endregion

		#region Execution Value
		public override object ExecutionValue
		{
			get
         { 
               return (object)0; 
         }
		}
		#endregion
		#endregion
		
      #region IDTSComponentPersist

		#region String Constants for node names
		private const string PERSIST_CONST_ELEMENT	               = "StartTaskData";
		private const string PERSIST_CONST_STRING                   = "StringValue";
		private const string PERSIST_CONST_INT                          = "IntValue";
		#endregion

		#region Save To XML
		public void SaveToXML(XmlDocument doc, IDTSInfoEvents events)
		{
         try
         {
            // Create an root node for the data
            XmlNode node = doc.CreateNode(XmlNodeType.Element, PERSIST_CONST_ELEMENT, "");
            XmlElement element = node as XmlElement;
            doc.AppendChild(node);

            // Persist the string value
            XmlAttribute attr = doc.CreateAttribute(PERSIST_CONST_STRING);
            attr.Value = m_privateString;
            element.Attributes.Append(attr);

            // Persist the int value
            attr = doc.CreateAttribute(PERSIST_CONST_INT);
            attr.Value = XmlConvert.ToString(m_privateInt);
            element.Attributes.Append(attr);
         }
         catch (Exception e)
         {
            if (null != events)
            {
               events.FireError(0, "", e.Message, "", 0);
            }
         }

		}
		#endregion         

      #region Load From XML
      public void LoadFromXML(XmlElement rootNode, IDTSInfoEvents events)
		{
         try
         {
			   //Set properties to their default values;
			   m_privateString = "";
			   m_privateInt = 0;

            // Load the root node for the data
            // This should be the root data node created above.
            // If not, throw and show. 
            if (rootNode.Name != PERSIST_CONST_ELEMENT)
				   throw new Exception("Invalid Start Task data node");

			   // Load the string value
			   XmlNode attr = rootNode.Attributes.GetNamedItem(PERSIST_CONST_STRING);
			   if (attr != null)
				   m_privateString = attr.Value;

			   // Load the int value
			   attr = rootNode.Attributes.GetNamedItem(PERSIST_CONST_INT);
			   if (attr != null)
				   m_privateInt = XmlConvert.ToInt32(attr.Value);
         }
         catch( Exception e)
         {
            if (null != events)
            {
               // Shows the exception as an error with the message.
               events.FireError(0, "", e.Message, "", 0);
            }          
         }
		}
		#endregion
      
      #endregion

      #region Logging
      // List of the log events that could potentially be logged from this task
      // Used to add log event entries into the LogEventInfos collection in InitializeTask
		private static string[] LOGEVENTNAMES = 
		{ 
			"TestLogEntry"			
		};

		// Enumerate values to index into the LOGEVENTNAMES above
		private enum ENUMLOG
		{
			TESTLOGENTRY
		}
      // A simplified method for logging.
      // Does not require the additional overwritten parameters.
      private void WriteLog(IDTSLogging log, string logEventName, string message)
      {
         if (log != null)
         {
            byte[] nothing = null;
            log.Log(logEventName, null, null, null, null, null, message, DateTime.Now, DateTime.Now, 0, ref nothing);
         }
      }
		#endregion
   }
}
