using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using DTS = Microsoft.SqlServer.Dts.Runtime;
using Microsoft.DataTransformationServices.Controls;

namespace SSIS2K5.Samples.Utilities.ConfigBatch
{

   public partial class ConfigBatchWnd : Form
   {
      #region PRIVATES
      private DTS.DTSConfigurationType m_configurationType = DTS.DTSConfigurationType.ConfigFile;
      private string m_configurationString = "";
      private string m_packagePath = "";
      private string m_configurationName = "<Configuration Name>";
      private string m_configurationDescription = "<Configuration Description>";
      private string[] m_FileNames;

      private static int COUNTCONFIGTYPES = 11;
      private static string CONFIG_XML = "XML";
      private static string CONFIG_IXML = "Indirect XML";
      private static string CONFIG_PARENT = "Parent Package";
      private static string CONFIG_ENV = "Environment Variable";
      private static string CONFIG_REG = "Registry";
      private static string CONFIG_IPARENT = "Indirect Parent Package";
      private static string CONFIG_IREG = "Indirect Registry";
      private static string CONFIG_SQL = "SQL Server";
      private static string CONFIG_ISQL = "Indirect SQL Server";
      private static string CONFIG_INI = "INI File";
      private static string CONFIG_IINI = "Indirect INI File";

      private static string ERROR_CONFIGEXISTS = "Package already contains the configuration with the following name : ";
      private static string ERROR_CONFIGNOTEXISTS = "Package does not contain the configuration with the following name : ";

      CommitOp m_op = CommitOp.add;
      Hashtable typeTable = new Hashtable(COUNTCONFIGTYPES);
      
      enum CommitOp
      {
         add, update, delete, none
      }
      #endregion

      #region HELPER METHODS

      private void ClearLog()
      {
         rtLogWnd.Clear();
      }

      private void LogMessage(string msg)
      {
         rtLogWnd.Text += msg + "\r\n";
      }

      private void LogError(string msg)
      {
         rtLogWnd.Text += msg + "\r\n";         
         System.Diagnostics.Trace.TraceError(msg);
      }

      // Fill the type selection combobox
      private void PopulateTypeCombo()
      {
         // Add name and type to hash
         typeTable.Add(CONFIG_XML, DTS.DTSConfigurationType.ConfigFile);
         typeTable.Add(CONFIG_IXML, DTS.DTSConfigurationType.IConfigFile);         
         typeTable.Add(CONFIG_PARENT, DTS.DTSConfigurationType.ParentVariable);
         typeTable.Add(CONFIG_ENV, DTS.DTSConfigurationType.EnvVariable);
         typeTable.Add(CONFIG_REG, DTS.DTSConfigurationType.RegEntry);
         typeTable.Add(CONFIG_IPARENT, DTS.DTSConfigurationType.IParentVariable);
         typeTable.Add(CONFIG_IREG, DTS.DTSConfigurationType.IRegEntry);
         typeTable.Add(CONFIG_SQL, DTS.DTSConfigurationType.SqlServer);
         typeTable.Add(CONFIG_ISQL, DTS.DTSConfigurationType.ISqlServer);
         typeTable.Add(CONFIG_INI, DTS.DTSConfigurationType.IniFile);
         typeTable.Add(CONFIG_IINI, DTS.DTSConfigurationType.IIniFile);


         // Fill the combobox
         cbConfigType.Items.Add( CONFIG_XML );
         cbConfigType.Items.Add(CONFIG_IXML);
         cbConfigType.Items.Add(CONFIG_PARENT);
         cbConfigType.Items.Add(CONFIG_ENV);
         cbConfigType.Items.Add(CONFIG_REG);
         cbConfigType.Items.Add(CONFIG_IPARENT);
         cbConfigType.Items.Add(CONFIG_IREG);
         cbConfigType.Items.Add(CONFIG_SQL);
         cbConfigType.Items.Add(CONFIG_ISQL);
         cbConfigType.Items.Add(CONFIG_INI);
         cbConfigType.Items.Add(CONFIG_IINI);

         cbConfigType.SelectedIndex = 10;
      }

      private Boolean CanEnableCommitButton()
      {
         // Updated this when other types supported.
         // Currently ignore PackagePath value.
         pbProgress.Visible = false;

         if (m_op == CommitOp.delete && tbConfigName.Text.Length > 0)
            return true;

         return
            tbConfigString.Text.Length > 0 &&            
            tbConfigName.Text.Length > 0 &&
            tbPackagesList.Text.Length > 0;

      }

      private void EnableCommit()
      {
         if (CanEnableCommitButton())
            btCommit.Enabled = true;
         else
            btCommit.Enabled = false;
      }
      


      // Give initial values for the text boxes
      private void InitializeTextBoxes()
      {
         tbConfigDescription.Text = "Configuration added by the Configuration Batch Utility";
         tbConfigName.Text = "BatchConfigurationEntry";
         tbConfigString.Text = "";
         tbPackagePath.Text = "";
      }

      // Modify controls to be enabled/disabled according to selected options
      private void MorphInputControls()
      {
         if ( m_op == CommitOp.delete )
         {
            cbConfigType.Enabled = false;
            lConfigString.Text = "Configuration String";
            btFindConfigString.Visible = false;
            tbConfigString.Enabled = false;
            tbPackagePath.Enabled = false;
            tbConfigDescription.Enabled = false;
            return;
         }


         tbConfigString.Text = "";
         tbPackagePath.Text = "";
         cbConfigType.Enabled = true;
         tbConfigString.Enabled = true;
         tbConfigDescription.Enabled = true;

         // Enable and hide controls based on configuration type
         // Some configuration types don't use the package path value, disable then.
         // Some configuration types need a file or environment variable, show the button.
         switch ((int)m_configurationType)
         {
            case (int)DTS.DTSConfigurationType.ConfigFile:
               lConfigString.Text = "Configuration File Name";
               btFindConfigString.Visible = true;
               tbPackagePath.Enabled = false;
               break;
            case (int)DTS.DTSConfigurationType.IConfigFile:
               lConfigString.Text = "Environment Variable";
               btFindConfigString.Visible = true;
               tbPackagePath.Enabled = false;
               break;
            case (int)DTS.DTSConfigurationType.EnvVariable:
               lConfigString.Text = "Environment Variable";
               btFindConfigString.Visible = true;
               tbPackagePath.Enabled = true;
               break;
            case (int)DTS.DTSConfigurationType.IIniFile:
               lConfigString.Text = "Environment Variable";
               btFindConfigString.Visible = true;
               tbPackagePath.Enabled = false;
               break;
            case (int)DTS.DTSConfigurationType.IniFile:
               lConfigString.Text = "INI File";
               btFindConfigString.Visible = true;
               tbPackagePath.Enabled = false;
               break;
            case (int)DTS.DTSConfigurationType.IParentVariable:
               lConfigString.Text = "Environment Variable";
               btFindConfigString.Visible = true;
               tbPackagePath.Enabled = true;
               break;
            case (int)DTS.DTSConfigurationType.IRegEntry:
               lConfigString.Text = "Environment Variable";               
               btFindConfigString.Visible = true;
               tbPackagePath.Enabled = true;
               break;
            case (int)DTS.DTSConfigurationType.ISqlServer:
               lConfigString.Text = "Environment Variable";               
               btFindConfigString.Visible = true;
               tbPackagePath.Enabled = false;
               break;
            case (int)DTS.DTSConfigurationType.ParentVariable:
               lConfigString.Text = "Parent Variable Name or ID";               
               btFindConfigString.Visible = false;
               tbPackagePath.Enabled = true;
               break;
            case (int)DTS.DTSConfigurationType.RegEntry:
               lConfigString.Text = "Registry Branch";               
               btFindConfigString.Visible = false;
               tbPackagePath.Enabled = true;
               break;
            case (int)DTS.DTSConfigurationType.SqlServer:
               lConfigString.Text = "Configuration String";
               btFindConfigString.Visible = false;
               tbPackagePath.Enabled = false;
               break;
            default:
               return;
         }
      }

      #endregion

      #region CONSTRUCTOR
      public ConfigBatchWnd()
      {
         // System init
         InitializeComponent();

         // Setup the initial text values
         InitializeTextBoxes();

         // Private init
         PopulateTypeCombo();         
      }
      #endregion

      #region EVENTHANDLERS

      // This is where all the configurations are added, updated or deleted to/on/from packages
      // Commit changes to the selected packages
      private void CommitButton_Click(object sender, EventArgs e)
      {
         // Get the various configuration properties from the controls
         DTS.DTSConfigurationType
         m_configurationType =  (DTS.DTSConfigurationType) typeTable[cbConfigType.SelectedItem.ToString()];
         m_configurationString = tbConfigString.Text;
         m_packagePath = tbPackagePath.Text;
         m_configurationName = tbConfigName.Text;
         m_configurationDescription = tbConfigDescription.Text;

         // Start a new log
         ClearLog();

         // Iterate through each of the packages and apply the configuration operation 
         try
         {
            // Are there package files to operate on?
            if (m_FileNames != null && m_FileNames.Length > 0)
            {
               // Initialize the progress bar and set the style, value and initial value
               pbProgress.Visible = true;               
               pbProgress.Style = ProgressBarStyle.Blocks;
               pbProgress.Value = 0;               

               // The increment used to represent each package
               int progLen = m_FileNames.Length + 1;

               // Start at 1 so never dividing by zero
               int currentCount = 1;

               
               // Change the cursor to an hourglass
               this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
               
               // Get an application opject for saving later
               DTS.Application app = new DTS.Application();

               // Process each package
               foreach (string fileName in m_FileNames)
               {
                  // Log which package processed
                  LogMessage("Processing the following package : " + fileName);
                                    
                  try
                  {
                     // Load the package
                     DTS.Package p;

                     // Load the package with the password if it needs it.
                     // PackageProtectionUtils is found in MS.DTS.Controls
                     string pw = "";
                     p = PackageProtectionUtils.LoadPackageFromFile(fileName, null, out pw);

                     // Get the Configurations collection from the current package
                     DTS.Configurations configs = p.Configurations;
                    
                     // Process depends on the type of operation selected controlled by radio buttons
                     switch (m_op)
                     {
                        // Add a new configuration unless configuration with the same name already exists
                        case CommitOp.add:
                           if (configs.Contains(m_configurationName))
                           {
                              LogError(ERROR_CONFIGEXISTS);
                              LogError(m_configurationName);
                           }
                           // Add it
                           else 
                           {
                              DTS.Configuration newConfig = configs.Add();
                              newConfig.ConfigurationString = m_configurationString;
                              newConfig.ConfigurationType = m_configurationType;
                              newConfig.Name = m_configurationName;
                              newConfig.Description = m_configurationDescription;
                              newConfig.PackagePath = m_packagePath;
                              LogMessage("Added the configuration named : " + m_configurationName + " to " + fileName);
                           }
                           break;
                        // Delete the configuration by name if it exists
                        case CommitOp.delete:
                           if (!configs.Contains(m_configurationName))
                           {
                              LogError(ERROR_CONFIGNOTEXISTS);
                              LogError(m_configurationName);
                           }
                           else 
                           {
                              configs.Remove(m_configurationName);
                              LogMessage("Removed the configuration named : " + m_configurationName + " from " + fileName);
                           }
                           break;
                        // Update an existing configuration by name if it exists
                        case CommitOp.update:
                           if (!configs.Contains(m_configurationName))
                           {
                              LogError(ERROR_CONFIGNOTEXISTS);
                              LogError(m_configurationName);
                           }
                           else
                           {
                              DTS.Configuration oldConfig = configs[m_configurationName];
                              oldConfig.ConfigurationString = m_configurationString;
                              oldConfig.ConfigurationType = m_configurationType;
                              oldConfig.Name = m_configurationName;
                              oldConfig.Description = m_configurationDescription;
                              oldConfig.PackagePath = m_packagePath;
                              LogMessage("Updated the configuration named : " + m_configurationName + " in " + fileName);
                           }
                           break;
                     }

                     // Save the package
                     app.SaveToXml( fileName, p, null );

                     // Update the progress bar and count
                     pbProgress.Value = (100 / progLen) * currentCount;
                     currentCount++;
                  }
                  catch(Exception ex)
                  {
                     LogError(ex.Message);
                  }
               } 
            }                                   
         }
         catch (Exception ex)
         {
            LogError(ex.Message);
         }
         finally
         {
            // Cleanup
            pbProgress.Value = 100;
            this.Cursor = System.Windows.Forms.Cursors.Arrow;
         }
      }

      // Just close the dialog box
      private void OKButton_Click(object sender, EventArgs e)
      {
         this.Close();
      }

      // User is choosing a different configuration type
      private void cbConfigType_SelectedIndexChanged(object sender, EventArgs e)
      {
         // Get the name
         string index = cbConfigType.SelectedItem.ToString();

         // Find the type from the name
         m_configurationType = (DTS.DTSConfigurationType)typeTable[index];

         // Update the user controls to reflect the configuration type requirements
         MorphInputControls();
      }

      private void btFindConfigString_Click(object sender, EventArgs e)
      {
         // If an indirect type,
         if (m_configurationType == DTS.DTSConfigurationType.IConfigFile ||
             m_configurationType == DTS.DTSConfigurationType.IIniFile ||
             m_configurationType == DTS.DTSConfigurationType.IParentVariable ||
             m_configurationType == DTS.DTSConfigurationType.IRegEntry ||
             m_configurationType == DTS.DTSConfigurationType.ISqlServer ||
             m_configurationType == DTS.DTSConfigurationType.EnvVariable)
         {
            // find an environment variable
            EnvVarWnd envVarWnd = new EnvVarWnd();
            if (DialogResult.OK == envVarWnd.ShowDialog())
            {
               tbConfigString.Text = envVarWnd.selectedEnvVar;
            }
         }
         // If an INI file configuration type
         else if (m_configurationType == DTS.DTSConfigurationType.IniFile)
         {
            // find an INI file.
            if (DialogResult.OK == openINIDialog.ShowDialog())
            {
               tbConfigString.Text = openINIDialog.FileName;
            }
         }
         else
         {
            // Otherwise, find a configuration file.
            if (DialogResult.OK == openConfigDialog.ShowDialog())
            {
               tbConfigString.Text = openConfigDialog.FileName;
            }
         }
         // TODO : Launch browser for SQL Type and RegEdit.exe for registry type.
      }

      // The packages list changed, update controls
      private void tbPackagesList_TextChanged(object sender, EventArgs e)
      {
         if (tbPackagesList.Text.Length > 0)
         {
            btAddPackages.Enabled = true;
         }
         else
         {
            btAddPackages.Enabled = false;
         }

         EnableCommit();
      }

      // The ConfigString text changed, update controls
      private void tbConfigString_TextChanged(object sender, EventArgs e)
      {
         EnableCommit();
      }

      // The PackagePath text changed, update controls
      private void tbPackagePath_TextChanged(object sender, EventArgs e)
      {
         EnableCommit();
      }

      // The ConfigName text changed, update controls
      private void tbConfigName_TextChanged(object sender, EventArgs e)
      {
         EnableCommit();
      }

      // The Update radio button changed, update controls
      private void rbUpdate_CheckedChanged(object sender, EventArgs e)
      {
         if (rbUpdate.Checked)
            m_op = CommitOp.update;
         MorphInputControls();
         EnableCommit();
      }

      // The Delete radio button changed, update controls
      private void rbDelete_CheckedChanged(object sender, EventArgs e)
      {
         if (rbDelete.Checked)
            m_op = CommitOp.delete;
         MorphInputControls();
         EnableCommit();
      }

      // The Add radio button changed, update controls
      private void rbAdd_CheckedChanged(object sender, EventArgs e)
      {
         if (rbAdd.Checked)
            m_op = CommitOp.add;
         MorphInputControls();
         EnableCommit();
      }

      // Browse for packages to modify.
      private void btFindPackages_Click(object sender, EventArgs e)
      {
         // Open the find packages dialog.
         if (DialogResult.OK == openPackagesDialog.ShowDialog())
         {
            // Get the file names.
            m_FileNames = openPackagesDialog.FileNames;
            if (m_FileNames.Length > 0)
            {
               // Add them together into one string
               StringBuilder sbPackages = new StringBuilder();

               // Format them and add them to the packages to modify textbox
               foreach (string packageName in openPackagesDialog.FileNames)
               {
                  sbPackages = sbPackages.Append(packageName + " | ");
               }

               // Get rid of the last |
               sbPackages.Remove(sbPackages.Length - 2, 2);

               // Add the package names to the edit box
               tbPackagesList.Text = sbPackages.ToString();
            }
         }
      }

      // Browse for more packages to add.
      // This feature allows the user to select packages from more than one location
      private void btAddPackages_Click(object sender, EventArgs e)
      {
         // Get the packages to add
         if (DialogResult.OK == openPackagesDialog.ShowDialog())
         {
            // Retain the old packages and add new ones to them.
            string[] tempNames = new string[m_FileNames.Length + openPackagesDialog.FileNames.Length];
            m_FileNames.CopyTo(tempNames, 0);
            openPackagesDialog.FileNames.CopyTo(tempNames, m_FileNames.Length);
            m_FileNames = tempNames;

            StringBuilder sbPackages = new StringBuilder(tbPackagesList.Text);

            // If both have text, add a '|' character
            if (tbPackagesList.Text.Length > 0 && m_FileNames.Length > 0)
            {
               sbPackages.Append(" |");
            }

            // Add the additional packages
            foreach (string packageName in openPackagesDialog.FileNames)
            {
               sbPackages = sbPackages.Append(packageName + " | ");
            }

            // Get rid of the last |
            sbPackages.Remove(sbPackages.Length - 2, 2);

            // Add the package names to the edit box
            tbPackagesList.Text = sbPackages.ToString();
         }
      }

      // Show an about box.
      private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
      {
         AboutBox about = new AboutBox();
         about.ShowDialog();
      }

      #endregion

  }
}